clear
close all
clc

%This program solves the counterfactual model where owner hours are
%restricted to be larger than 40 (Table 5 and Figures 7+8)
%% 

% The code is written for a more general CES production function than used in the paper: [lambda h^rho+(1-lambda)L^rho]^(1/rho) where we set lambda=0.5. 
% As a consequence, mean firm productivities are rescaled.
% See main.m for more detailed comments on the program.

mue = 0.8900; %mean log firm productivity. Rescale:  mue+log(lambda^((1-alpha)*eta/elas));
sigma = 0.1420; %std log firm productivity 
rho = 0.987;      %autocorrelation of log productivity     
xi=0.0;          %fixed costs of labor (not present in the paper)                                 
eta = 0.88;      %returns of scale 
theta =0.244;     %financial friction (firm growth) 
nu.mean = -1.05; %average log disutility 
nu.rho = 0.47;   %autocorrelation of log leissure   
nu.std = 0.9;   %std of log disitulity 
kappaS=0.445; %productivity scale of self-employed. Rescale: kappaS^(lambdaS*eta)/(lambda^((1-alpha)*eta/elas))
lambdaS = 0.61; %returns of scale of self-employed
prob_w = 0.350; %possibility to choose being a worker
sigma_w = 0.29; %standard deviation of shock to labor income
beta = 0.975; %discount factor
lambda = 0.5; %
lambda2 = 1-lambda;
elas=0.5; %parameter of CES function


%worker productivity
mu_w = 0;
rho_w = 0.95;
nw = 5;
sigma2.w = (sigma_w^2)/(1-rho_w^2);
[grid_w,P_w] = Tauchen(rho_w,nw,sigma2.w,mu_w,'importance'); 
grid_w = exp(grid_w)';
prob_become_w = ones(nw,1)/nw;
prob_become_wcum = cumsum(prob_become_w);
P_wcum = cumsum(P_w,2);

phi = 2.0; %elasticity of labor supply
gamma = 1.5; %risk aversion
delta = 0.06; %depreciation rate
hmax=2.25;
nu.inov = nu.std;  


alpha = 0.33;
alpha2=1-alpha;

r = 0.00695; %guess for equilibrium interest rate


wage = (((alpha^alpha)*((1-alpha)^(1-alpha)))/(r+delta))^(1/(1-alpha));



%hours grid
nh = 6;
hgrid = linspace(1,hmax,nh)'; %hours restricted to be at least 40 (1)

%productivity grid
nz = 21;
sigma2.z = (sigma^2)/(1-rho^2); %long-run variance
[Z,P] = Tauchen_asy(rho,nz,sigma2.z,mue);
% [Z,P] = Tauchen(rho,nz,sigma2.z,mue,'equi');
z_min = min(Z);
z_max = max(Z);
log_Z=Z;
Z = exp(Z)';
Ptran = cumsum(P,2);


%grid of preferences
nnu = 6;
sigma2.nu = (nu.std^2)/(1-nu.rho^2);
[nu.grid,nu.P] = Tauchen(nu.rho,nnu,sigma2.nu,nu.mean,'importance'); 
nu.grid=nu.grid';
nu.tran = cumsum(nu.P,2);
nu_min = min(nu.grid);
nu_max = max(nu.grid);

nu.grid_log = nu.grid;
nu.min = min(nu.grid);
nu.max = max(nu.grid);
nu.grid = exp(nu.grid);


%compute largest possible firm (optimal unconstrained choice):
lgrid = linspace(0,500,400)';
Xgrid = (lambda*(hmax^elas)+lambda2*(lgrid.^elas)).^(1/elas);

aux1=alpha*eta/(r+delta);
aux3 = 1/(1-alpha*eta);
aux4 = alpha2*eta/(1-alpha*eta);
aux5 = elas/(1-elas);

Yaux = (Z(end)*(aux1^(alpha*eta))*Xgrid.^(alpha2*eta)).^aux3;
Kaux = Yaux*aux1;
PIaux = Yaux-wage*lgrid-(r+delta)*Kaux;
[~,index] = max(PIaux);
Y_max=Yaux(index);
K_max = Kaux(index);
L_max = lgrid(index)

if L_max==max(lgrid)
    xyz %increase upper bound on labor
end

%asset grid
amin = 0.01;
amax = max(6*wage,(K_max*(r+delta)+wage*L_max+xi)/theta);
na = 244;
agrid = [amin; wage/15; wage/10; wage/9; wage/8; wage/7; wage/6; wage/5; wage/4; wage/3; wage/2; wage; wage*1.5; wage*2; wage*2.5; wage*3; wage*3.5; wage*4; wage*5; wage*6; wage*7;];
aux = linspace(wage*7,amax/10,(na/2)-19)';
agrid = [agrid; aux(2:end)];
aux = linspace(amax/10,amax,ceil(na/2+1))';
agrid = [agrid; aux(2:end)];
na = na+1;

%grid of asset choices (larger than agrid)
diffa=diff(agrid);
diffa2=diffa/9;
agridp=zeros(9*na-8,1);
for i =1:na-1
    agridp(1+9*(i-1):9*i,1) = agrid(i)+(0:8)'*diffa2(i);
end
agridp(end)=agrid(end);
nap = length(agridp);
[~,xia] = histc(agridp,agrid); %find lower bounds on asset grid
xia(agridp<agrid(1)) = 1;
xia(agridp>=agrid(end)) = na-1;
sia = (agridp - agrid(xia))./diffa(xia);

lgrid = linspace(0,L_max,400)';

%auxilary variables
sub = 10^(-100);

%% solve for posible static choices


L = zeros(na,nz,nh,4);
K = zeros(na,nz,nh,4);
Yin = zeros(na,nz,nh,4);
Cin = zeros(na,nz,nh,4);

%loop over states
for i = 1:na
    aux6 = theta*agrid(i)-xi;
    for j = 1:nz
        for i2 = 1:nh %loop of hours choice
            h = hgrid(i2);
            
            Xgrid = (lambda*(h^elas)+lambda2*(lgrid.^elas)).^(1/elas);
            %case 1: L>0 FC do not bind
            if aux6>0.0001
                Yaux = (Z(j)*(aux1^(alpha*eta))*Xgrid.^(alpha2*eta)).^aux3;
                Kaux = Yaux*aux1;
                upper = (aux6-wage*lgrid)/(r+delta);
                index = Kaux>upper;
                Yaux(index) = 0;
                PIaux = Yaux-wage*lgrid-(r+delta)*Kaux;
                [~,index] = max(PIaux);
                Yin(i,j,i2,1)=Yaux(index);
                K(i,j,i2,1)=Kaux(index);
                L(i,j,i2,1) = lgrid(index);
                Cin(i,j,i2,1)=(r+delta)*Kaux(index)+wage*lgrid(index);
            else
                Yin(i,j,i2,1)=-1000;
            end
            
            %case 2: L>0, FC bind
            if aux6>0.0001
                Kaux = (aux6-wage*lgrid)/(r+delta);
                Kaux(Kaux<0) = 0;
                Yaux = Z(j)*(Kaux.^(alpha*eta)).*(Xgrid.^(alpha2*eta));
                PIaux = Yaux-wage*lgrid-(r+delta)*Kaux;
                [~,index] = max(PIaux);
                Yin(i,j,i2,2)=Yaux(index);
                K(i,j,i2,2)=Kaux(index);
                L(i,j,i2,2) = lgrid(index);
                Cin(i,j,i2,2)=(r+delta)*Kaux(index)+wage*lgrid(index);
            else
                Yin(i,j,i2,2)=-1000;
            end
            
            if h==0
                Yin(i,j,i2,3)=-1000;
                Yin(i,j,i2,4)=-1000;
            else
                %case 3: lm=0,lb=0, FC no bind
                y=(Z(j)*(aux1^(alpha*eta))*((kappaS*h)^(lambdaS*eta)))^aux3;
                kk = aux1*y;
                upper = (theta*agrid(i))/(r+delta);
                if (kk/upper>1.005 && (kk-upper)>0.01)
                    y=0;
                end
                Cin(i,j,i2,3) = (r+delta)*kk;
                Yin(i,j,i2,3)=y;
                K(i,j,i2,3)=kk;
                L(i,j,i2,3) = 0;
                
                %case 6: lm=0,lb=0 FC bind
                y=Z(j)*(((aux6+xi)/(r+delta))^(alpha*eta))*((kappaS*h)^(lambdaS*eta));
                kk = (aux6+xi)/(r+delta);
                Cin(i,j,i2,4) = (r+delta)*kk;
                Yin(i,j,i2,4)=y;
                K(i,j,i2,4)=kk;
                L(i,j,i2,4) = 0;
            end
        end
    end
end

%% VFI

%consumption choices of workers
consumew = (1+r)*repmat(agrid,[1 nap nw])-repmat(agridp',[na 1 nw])+wage*repmat(reshape(grid_w,[1 1 nw]),[na nap 1]);
consumew = max(consumew,sub);
consumew = log(consumew);

%disutility of working for workers and owners
hwutil = ((1.^(1+phi))./(1+phi));
heutil = ((hgrid.^(1+phi))./(1+phi));

%firm profits
PI=Yin-Cin;
PImax = zeros(na,nz);
for i =1:na
    for j = 1:nz
        PImax(i,j) = max(max(max(PI(i,j,:,:))));
    end
end

%static optimal labor choice
[PI_opt,index_l] = max(PI,[],4);


%guess labor and capital policy functions and value function
load('V_counter.mat','V_counter')
load('Vw_counter.mat','Vw_counter')

Vold_e = V_counter;
Vold_w = Vw_counter;

% Vold_e = zeros(na,nz,nnu,nw);
% Vold_w = zeros(na,nz,nnu,nw);


crit=1;
while crit>5*10^(-3)
    
    %employment decisions
    Echoice = Vold_e>Vold_w;
    
    EVE_e = (prob_w*max(Vold_e,Vold_w)+(1-prob_w)*Vold_e);
    EVE_w = max(Vold_e,Vold_w);
    
    %worker
    [Vw,index_w_h,index_w_a] = VFI_w_mex(EVE_w,consumew,int32(na),int32(nap),int32(nz),int32(nnu),int32(nh),int32(nw),agrid,agridp,nu.grid,hgrid,beta,gamma,phi,sub,r,wage,P,nu.P,int32(xia),sia,hwutil,P_w);
    
    %owners
    [V,index_a,index_h] = VFI_e4_mex(EVE_e,int32(na),int32(nap),int32(nz),int32(nnu),int32(nh),int32(nw),agrid,agridp,nu.grid,beta,gamma,sub,r,P,nu.P,P_w,int32(xia),sia,heutil,PI,PImax,PI_opt,hgrid,hmax);
    
    
    crit1 = max(max(max(max(abs(V-Vold_e)))));
    crit2 = max(max(max(max(abs(Vw-Vold_w)))));
    crit = max([crit1 crit2]);
    Vold_e = V;
    Vold_w = Vw;
    
end
V_counter=Vold_e;
Vw_counter=Vold_w;
save('V_counter.mat','V_counter')
save('Vw_counter.mat','Vw_counter')


%policies
lchoice = zeros(na,nz,nnu,nw); %labor choice
hchoice = zeros(na,nz,nnu,nw); %hours choice
kchoice = zeros(na,nz,nnu,nw); %capital choice
aprime_e = zeros(na,nz,nnu,nw); %asset choice owners
hchoice_w = zeros(na,nz,nnu,nw); %hours choice workers=1
aprime_w = zeros(na,nz,nnu,nw); %asset choice workers
for i = 1:na
    for l = 1:nnu
        for j = 1:nz
            for k =1:nw
                aprime_e(i,j,l,k) = agridp(index_a(i,j,l,k));
                hchoice(i,j,l,k) = hgrid(index_h(i,j,l,k));
                lchoice(i,j,l,k) = L(i,j,index_h(i,j,l,k),index_l(i,j,index_h(i,j,l,k)));
                kchoice(i,j,l,k) = K(i,j,index_h(i,j,l),index_l(i,j,index_h(i,j,l,k)));
                hchoice_w(i,j,l,k) = 1;
                aprime_w(i,j,l,k) = agridp(index_w_a(i,j,l,k));
            end
        end
    end
end


Y = output_ces(alpha,lambda,eta,elas,repmat(reshape(Z,[1 nz 1]),[na 1 nnu nw]),kchoice,lchoice,hchoice);
for i=1:na
    for j =1:nz
        for k = 1:nnu
            for l = 1:nw
                if (lchoice(i,j,k,l))==0
                    Y(i,j,k,nw) = Z(j)*(kchoice(i,j,k,l).^(alpha*eta))*(kappaS*(hchoice(i,j,k,l)))^(lambdaS*eta);
                end
            end
        end
    end
end
Costs = (r+delta)*kchoice+wage*lchoice+(lchoice>0)*xi;


%profits and consumption
PII = Y-Costs;
CC_e = zeros(na,nz,nnu,nw);
CC_w = zeros(na,nz,nnu,nw);
for i = 1:na
    for l = 1:nnu
        for j = 1:nz
            for k = 1:nw
                CC_e(i,j,l,k) = (1+r)*agrid(i)-aprime_e(i,j,l,k)+PII(i,j,l,k);
                CC_w(i,j,l,k) = (1+r)*agrid(i)-aprime_w(i,j,l,k)+hchoice_w(i,j,l,k)*wage*grid_w(k);
            end
        end
    end
end



%% Simulation
H=10000; %number of individuals
T=500; %time periods
rand('seed',262524);
randn('seed',276624);

z_shock = sigma*randn(H,T);
nu_shock = nu.std*randn(H,T);
job_shock = rand(H,T);
w_shock = rand(H,T);


%initialization
load('panel_ergo.mat','panel_ergo')
panel = zeros(H,12,100); %prod, nu, A, w, C, Y, h, l, K, type1, age, ID

panel(:,12,1) =1:H;
panel(:,1,1) =panel_ergo(:,1);
panel(:,2,1) =panel_ergo(:,2);
panel(:,3,1) =panel_ergo(:,3);
panel(:,4,1) =panel_ergo(:,4);
panel(:,10,1) =panel_ergo(:,10);
panel(panel_ergo(:,10)==2,11,1) =1;



for t1=1:T-1 %keep simulating
    
    %lower grid points for state
    [~,xii] = histc(panel(:,3,t1),agrid); %find lower bounds assets
    xii(panel(:,3,t1)<=agrid(1)) = 1;
    xii(panel(:,3,t1)>=agrid(end)) = na-1;
    [~,yii] = histc(panel(:,1,t1),log_Z); %find lower bounds assets
    yii(panel(:,1,t1)<=log_Z(1)) = 1;
    yii(panel(:,1,t1)>=log_Z(end)) = nz-1;
    [~,zii] = histc(panel(:,2,t1),nu.grid_log); %find lower bounds assets
    zii(panel(:,2,t1)<=nu.grid_log(1)) = 1;
    zii(panel(:,2,t1)>=nu.grid_log(end)) = nnu-1;
    
    %productivity next period
    panel(:,1,t1+1) = min(z_max+0.08,max(z_min-0.08,rho*panel(:,1,t1)+(1-rho)*mue+z_shock(:,t1)));
    
    %leissure next period
    panel(:,2,t1+1) = min(nu_max+0.2,max(nu_min-0.2,nu.rho*panel(:,2,t1)+(1-nu.rho)*nu.mean+nu_shock(:,t1)));
    
    for i1 = 1:H
        %asset,labor,hours choice
        if panel(i1,10,t1) == 1 %worker
            panel(i1,7,t1) = 1;
            panel(i1,3,t1+1) = interp3_l(agrid,log_Z,nu.grid_log,aprime_w(:,:,:,panel(i1,4,t1)),panel(i1,3,t1),panel(i1,1,t1),panel(i1,2,t1),xii(i1),yii(i1),zii(i1));
            panel(i1,6,t1) = wage;
        else
            panel(i1,7,t1) = max(0,interp3_l(agrid,log_Z,nu.grid_log,hchoice(:,:,:,panel(i1,4,t1)),panel(i1,3,t1),panel(i1,1,t1),panel(i1,2,t1),xii(i1),yii(i1),zii(i1)));
            panel(i1,8,t1) = max(0,interp3_l(agrid,log_Z,nu.grid_log,lchoice(:,:,:,panel(i1,4,t1)),panel(i1,3,t1),panel(i1,1,t1),panel(i1,2,t1),xii(i1),yii(i1),zii(i1)));
            panel(i1,9,t1) = max(0,interp3_l(agrid,log_Z,nu.grid_log,kchoice(:,:,:,panel(i1,4,t1)),panel(i1,3,t1),panel(i1,1,t1),panel(i1,2,t1),xii(i1),yii(i1),zii(i1)));
            panel(i1,3,t1+1) = max(0,interp3_l(agrid,log_Z,nu.grid_log,aprime_e(:,:,:,panel(i1,4,t1)),panel(i1,3,t1),panel(i1,1,t1),panel(i1,2,t1),xii(i1),yii(i1),zii(i1)));
            if panel(i1,8,t1)>0
                panel(i1,6,t1) = output_ces(alpha,lambda,eta,elas,exp(panel(i1,1,t1)),panel(i1,9,t1),panel(i1,8,t1),panel(i1,7,t1));
            else
                panel(i1,6,t1) = exp(panel(i1,1,t1))*((panel(i1,9,t1).^(alpha*eta)))*((kappaS*panel(i1,7,t1))^(lambdaS*eta));
            end
        end
    end
    
    %next period states
    [~,xii] = histc(panel(:,3,t1+1),agrid); %find lower bounds assets
    xii(panel(:,3,t1+1)<=agrid(1)) = 1;
    xii(panel(:,3,t1+1)>=agrid(end)) = na-1;
    [~,yii] = histc(panel(:,1,t1+1),log_Z); %find lower bounds assets
    yii(panel(:,1,t1+1)<=log_Z(1)) = 1;
    yii(panel(:,1,t1+1)>=log_Z(end)) = nz-1;
    [~,zii] = histc(panel(:,2,t1+1),nu.grid_log); %find lower bounds assets
    zii(panel(:,2,t1+1)<=nu.grid_log(1)) = 1;
    zii(panel(:,2,t1+1)>=nu.grid_log(end)) = nnu-1;
    for i1 = 1:H
        %type transition
        panel(i1,4,t1+1) = sum(w_shock(i1,t1)>P_wcum(panel(i1,4,t1),:))+1;
        if panel(i1,10,t1) == 1 %worker
            echoice = interp3_l(agrid,log_Z,nu.grid_log,Echoice(:,:,:,panel(i1,4,t1+1)),panel(i1,3,t1+1),panel(i1,1,t1+1),panel(i1,2,t1+1),xii(i1),yii(i1),zii(i1));
            if echoice<=0.5
                panel(i1,10,t1+1) = 1;
            else
                panel(i1,10,t1+1) = 2;
                panel(i1,11,t1+1) = 1; 
            end
        else
            if job_shock(i1,t1)<prob_w %can become worker
                echoice = interp3_l(agrid,log_Z,nu.grid_log,Echoice(:,:,:,panel(i1,4,t1+1)),panel(i1,3,t1+1),panel(i1,1,t1+1),panel(i1,2,t1+1),xii(i1),yii(i1),zii(i1));
                if echoice>0.5
                    panel(i1,10,t1+1) = 2;
                    panel(i1,11,t1+1) = panel(i1,11,t1)+1;
                else
                    panel(i1,10,t1+1) = 1;
                end
            else
                panel(i1,10,t1+1) = 2;
                panel(i1,11,t1+1) = panel(i1,11,t1)+1;
            end
        end
    end
end

panel(:,12,end) =1:H;
panel = panel(:,:,101:T);
for t1 = 1:T-100
    prod(t1,1) = mean(panel(:,1,t1));
    leis(t1,1) = mean(panel(:,2,t1));
    share_e(t1,1) = mean(panel(:,10,t1)==2);
    supply_a(t1,1) = sum(panel(:,3,t1));
    demand_a(t1,1) = sum(panel(:,9,t1));
    supply_w(t1,1) = sum(grid_w(panel(panel(:,10,t1)==1,4,t1)));
    demand_w(t1,1) = sum(panel(:,8,t1));
end

%equilibrium
net_supply.w = mean(supply_w-demand_w)/H;
net_supply.a = mean(supply_a-demand_a)/H


ratio1 = net_supply.w/net_supply.a;
alpha/(1-alpha)*wage*ratio1-delta %equilibrium interest rate. Should match guess


panel2 = zeros(H*(T-100),12);
for h = 1:H
    transform = squeeze(panel(h,:,1:end));
    transform = transform';
    panel2(1+(h-1)*(T-100):h*(T-100),:) = transform;
end
panel = panel2;
clear panel2


index = logical((panel(:,10)==2).*(panel(:,7)>0)); %idenify owners
share_e=mean(index); %share of owners
firms = panel(index==1,:);

index = firms(:,8)==0; %non-employers
employers = firms(index==0,:);
self = firms(index==1,:);
workers = panel(panel(:,10)==1,:);

%% Wealth inequality


%Owner wealth
top5_ent = prctile(firms(:,3),95);
top10_ent = prctile(firms(:,3),90);
top25_ent = prctile(firms(:,3),75);
top50_ent = prctile(firms(:,3),50);
top75_ent = prctile(firms(:,3),25);

aux= firms(firms(:,3)>amin,3);
wealth.entre.std=std(log(aux));
wealth.entre.r9075=top10_ent/top25_ent;
wealth.entre.r9050=top10_ent/top50_ent;
wealth.entre.r7525=top25_ent/top75_ent;
wealth.entre.r7550=top25_ent/top50_ent;
wealth.entre.r5025=top50_ent/top75_ent;

%% Figures


%Figure 7
f7_base = csvread('f7_base.csv');
f7_40 = zeros(10,1);
f7_40(1,1) = sum(firms(:,3)<=f7_base(1,2))/f7_base(1,1); %number of owners active relative to baseline in first decile of wealth 
for i = 2:10
    f7_40(i,1) = sum((firms(:,3)<=f7_base(i,2)).*(firms(:,3)>f7_base(i-1,2)))/f7_base(i,1);
end
csvwrite('f7_40.csv',f7_40)

%Figure 8
f8_base = csvread('f8_base.csv');
f8_40 = zeros(10,1);
f8_40(1,1) = sum(firms(:,1)<=f8_base(1,2))/f8_base(1,1); %number of owners active relative to baseline in first decile of wealth
for i = 2:10
    f8_40(i,1) = sum((firms(:,1)<=f8_base(i,2)).*(firms(:,1)>f8_base(i-1,2)))/f8_base(i,1);
end
csvwrite('f8_40.csv',f8_40)
